
/* rectbl.h -- G. Cisneros, 4.91; rev. 2.10.91 */
/* Mandatory declarations */

# include "rec.h"

int r_lpar();		/* compilation of a left parenthesis 		*/
int r_rpar();		/* compilation of a right parenthesis 		*/
int r_colon();		/* compilation of a colon 			*/
int r_semicol();	/* compilation of a semicolon 			*/
int r_code();		/* compilation of a simple operator 		*/
int r_oper1();		/* compilation of an operator with an ASCII arg */
int r_pred();		/* compilation of a simple predicate		*/
int r_pred1();		/* compilation of a predicate with an ASCII arg */
int r_noopc();		/* compilation of no-op 			*/
int r_comment();	/* compilation of a (bracket enclosed) comment 	*/
int r_ubrack();		/* compilation of an unbalanced right bracket 	*/
int r_lbrace();		/* compilation of a (brace enclosed) program 	*/
int r_ubrace();		/* compilation of an unbalanced right brace 	*/

int r_call();		/* execution of predicate @x (call subr. x) 	*/
int r_quit();		/* execution of operator _ (exit to O.S.) 	*/
int r_xbrace();		/* execution of brace-enclosed program 		*/

/* Optional: Counter predicate */

 int r_ctrc();		/* compilation of a counter */
 int r_ctrx();		/* execution of a counter */

/* Optional declarations, needed only in versions using quotes and
   numbers (see cnum.c and cquo.c); r_ld, r_ldch and r_xstr must
   be provided by user */

// int r_dquote(); /* compile doubly quoted string into symbol table */
// int r_squote(); /* compile singly quoted char into the program array */
// int r_cmin();   /* compile '-' as simple operator, call r_cnum if followed 
//		   by digit or period */
// int r_cnum();   /* compile a number (WORD, LONG or REAL) into sym. table */

// int r_ld();		/* execution of compiled numbers */
// int r_ldch();	/* execution of apostrophe (single quote op.) */
// int r_xstr();	/* execution of quoted strings */

int r_cdblp(), r_ldblp();

/* Declarations of user-provided execution subroutines */

void ropla(), roplc(), roplf(), roplh(), ropll(), roplm(), ropln(), roplo();
void roplp(), ropls(), roplr();

void ropua(), ropuc(), ropuf(), ropuh(), ropuk(), ropul();
void ropum(), ropun(), ropuo();
void ropup(), ropur(), ropus(), roput(), ropuu(), ropux(), ropuz();
int  rpruq();

void ropqm();
int  rprns(), rprps();

int  rprze(), rpron(), rprtw(), rprth();

/* Table of compiling/executing function definitions,
   a pair for each printing ASCII character */

struct fptbl dtbl[] = {
	r_noopc,	FALSE,	" space (separator)",	/* [blank] */
	r_ctrc,		r_ctrx,	" !n! count to n   ",	/* ! */
//	r_dquote,	r_xstr,	"",	/* " quoted string */
	r_noopc,	FALSE,	"",	/* " quoted string */
	r_pred,		rprns,	" # - random predicate, probability 1/2  ", /* # */
	r_cdblp,	r_ldblp," $xxx.x$ - floating point number ",	/* $ */
	r_pred,		rprps,	" % - random predicate, probability 1/10 ", /* % */
	r_noopc,	FALSE,	" & ",						/* & */
//	r_squote,	r_ldch,	" quoted character",	/* ' */
	r_noopc,	FALSE,	"",	/* '  quoted char */
	r_lpar,		FALSE,	" ( - start of expression",	/* ( */
	r_rpar,		FALSE,	"  ) - end of expression",	/* ) */
	r_noopc,	FALSE,	"",	/* * */
	r_noopc,	FALSE,	" + ",	/* + */
	r_noopc,	FALSE,	" , ",	/* , separator */
//	r_cmin,		FALSE,	" - ",	/* - */
	r_noopc,	FALSE,	" - ",	/* - */
//	r_cnum,		r_ld,	" , ",	/* . f.p. number */
	r_noopc,	FALSE,	" . ",	/* . */
	r_noopc,	FALSE,	" / ",	/* / */
//	r_cnum,		r_ld,	" 0 ",	/* 0 number */
//	r_cnum,		r_ld,	" 1 ",	/* 1 number */
//	r_cnum,		r_ld,	" 2 ",	/* 2 number */
//	r_cnum,		r_ld,	" 3 ",	/* 3 number */
//	r_cnum,		r_ld,	" 4 ",	/* 4 number */
//	r_cnum,		r_ld,	" 5 ",	/* 5 number */
//	r_cnum,		r_ld,	" 6 ",	/* 6 number */
//	r_cnum,		r_ld,	" 7 ",	/* 7 number */
//	r_cnum,		r_ld,	" 8 ",	/* 8 number */
//	r_cnum,		r_ld,	" 9 ",	/* 9 number */
	r_pred,		rprze,	" 0 - switch ",	/* 0 */
	r_pred,		rpron,	" 1 - switch ",	/* 1 */
	r_pred,		rprtw,	" 2 - switch ",	/* 2 */
	r_pred,		rprth,	" 3 - switch ",	/* 3 */
	r_noopc,	FALSE,	" 4 ",	/* 4 number */
	r_noopc,	FALSE,	" 5 ",	/* 5 number */
	r_noopc,	FALSE,	" 6 ",	/* 6 number */
	r_noopc,	FALSE,	" 7 ",	/* 7 number */
	r_noopc,	FALSE,	" 8 ",	/* 8 number */
	r_noopc,	FALSE,	" 9 ",	/* 9 number */
	r_colon,	FALSE,	" : - repeat from opening lparen ",	/* : iteration */
	r_semicol,	FALSE,	" ; - exit at rparen with the value T ", /* ; true termination */
	r_noopc,	FALSE,	" < ",							/* < */
	r_noopc,	FALSE,	" = ",							/* = */
	r_noopc,	FALSE,	" > ",							/* > */
	r_code,		ropqm,	" ? - print register values",				/* ? */
	r_pred1,	r_call,	" @x - call subroutine x",				/* @ */
	r_code,		ropua,	" A - set the angle   [{angle=(r_dblpar*PI)/180.0;}] ",	/* A */
	r_noopc,	FALSE,	" B ",							/* B */
	r_code,		ropuc,	" C - draw a small circle ",				/* C */
	r_noopc,	FALSE,	" D ",							/* D */
	r_noopc,	FALSE,	" E ",							/* E */
	r_code, 	ropuf,	" F - increase the step length    [{scale/=sfac;}]",	/* F */
	r_noopc,	FALSE,	" G ",							/* G */
	r_code,		ropuh,	" H - double the step length ",				/* H */
	r_noopc,	FALSE,	" I ",							/* I */
	r_noopc,	FALSE,	" J ",							/* J */
	r_code,		ropuk,	" K - turn 60 deg left    [{angle+=PI/3.0;}]        ",	/* K */
	r_code,		ropul,	" L - turn 90 deg left    [{angle+=PI/2.0;}]        ",	/* L */
	r_code,		ropum,	" M - move without trace ",				/* M */
	r_code,		ropun,	" N - reflect in the next edge",			/* N */
	r_code,		ropuo,	" O - read top of polygon stack",			/* o */
	r_code,		ropup,	" P - push onto the polygon stack ",			/* P */
	r_pred,		rpruq,	" Q - quit on tiny  [{if (scale*high<5.0) T else F}]",	/* Q */
	r_oper1,	ropur,	" Rn - reflect in edge n",				/* R */
	r_code,		ropus,	" S - set the scale     ",				/* S */
	r_code,		roput,	" T - rhombus",						/* T */
	r_code,		ropuu,	" U - snub square",					/* U */
	r_noopc,	FALSE,	" V ",							/* V */
	r_noopc,	FALSE,	" W ",							/* W */
	r_oper1,	ropux,	" X - Xn places a regular n-gon at the origin",		/* X */
	r_noopc,	FALSE,	" Y ",							/* Y */
	r_code,		ropuz,	" Z - erase page",					/* Z */
	r_comment,	FALSE,	" [begin comment]",					/* [ */
	r_noopc,	FALSE,	"",							/* \ */
	r_ubrack,	FALSE,	" [end comment]   ",					/* ] */
	r_noopc,	FALSE,	"",							/* ^ */
	r_code,		r_quit,	"",							/* _ */
	r_noopc,	FALSE,	"",							/* ` */
	r_code,		ropla,	" a - set angle increment [{alpha=(r_dblpar*PI)/180.0;}]", /* a */
	r_noopc,	FALSE,	" b ",							/* b */
	r_oper1,	roplc,	" cx - draw circle in color x",				/* c */
	r_noopc,	FALSE,	" d ",							/* d */
	r_noopc,	FALSE,	" e ",							/* e */
	r_code,		roplf,	" f - reduce scale by a factor (slider) [{scale*=sfac;}]", /* f */
	r_noopc,	FALSE,	" g ",							/* g */
	r_code,		roplh,	" h - reduce the step by half                      ",	/* h */
	r_noopc,	FALSE,	" i ",							/* i */
	r_noopc,	FALSE,	" j ",							/* j */
	r_noopc,	FALSE,	" k ",							/* k */
	r_code,		ropll,	" l - veer to the left     [{angle+=afac*alpha;}]",	/* l */
	r_code,		roplm,	" m - move to a new point  [(x,y) = scale*high*(x,y)]",	/* m */
	r_code,		ropln,	" n - reflect in the previous edge",			/* n */
	r_code,		roplo,	" o - offset the polygon",				/* O */
	r_code,		roplp,	" p - pop the polygon stack  ",				/* p */
	r_noopc,	FALSE,	" q ",							/* q */
	r_code,		roplr,	" r - veer to the right        [{angle-=afac*alpha;}]",	/* r */
	r_code,		ropls,	" s - modify scale - $2.0$s doubles [{scale*=r_dblpar;}]",/* s */
	r_noopc,	FALSE,	" t ",							/* t */
	r_noopc,	FALSE,	" u ",							/* u */
	r_noopc,	FALSE,	" v ",							/* v */
	r_noopc,	FALSE,	" w ",							/* w */
	r_noopc,	FALSE,	" x ",							/* x */
	r_noopc,	FALSE,	" y ",							/* y */
	r_noopc,	FALSE,	" z ",							/* z */
	r_lbrace,	r_xbrace," { - start of program",				/* { */
	r_noopc,	FALSE,	"",							/* | */
	r_ubrace,	FALSE,	" } - end of program",					/* } */
	r_noopc,	FALSE,	""							/* ~ */
};
